// Процентная ставка по кредиту
var c_rate = 0.13;
// Процентная ставка страхования
var c_ins_rate = 0.015;

// Прожиточный минимум, в рублях
var c_minimum = 4900;

// Округление до второго знака
function round2(x) { return Math.round(100 * x) / 100; }
// Округление до четвертого знака
function round4(x) { return Math.round(10000 * x) / 10000; }

/*
 * "Имущественное страхование и страхование жизни"
 * XLS: =Amount*1,1*C57/12
 */
function calc_ins(amount, ins_rate)
{
	return round2(amount * 1.1 * ins_rate / 12);
}

/*
 * "Платежи по жилищному кредиту (займу)"
 * XLS: =ROUND(Amount*Rate/12/(1-(1+Rate/12)^(2-Term));2)
 */
function calc_annual(amount, term, rate)
{
	return round2(amount * (rate / 12) / (1 -  Math.pow((1 + rate/12), (2 - term))));
}

/*
 * Обратная функция для вычисления суммы кредита по известному ежемесячному платежу
 * (включая страховку)
 */
function calc_rev_amount(payment, term, rate, ins_rate)
{
	return round2(
		payment / (
			(1.1 * ins_rate / 12) + ((rate / 12) / (1 - Math.pow((1 + rate/12), (2 - term))))
		)
	);
}

/*
 * Обратная для вычисления срока кредита по известной сумме и ежемесячному платежу
 * (включая страховку)
 */
function calc_rev_term(amount, payment, rate, ins_rate)
{
	var annual = payment - calc_ins(amount, ins_rate);
	return round2 (2 - Math.log(1 - amount * (rate / 12) / annual) / Math.log(1 + rate/12));
}

/*
 * Вычисление норматива Платеж/Доход
 * XLS: =IF(KZ<=0,5;0,5;(25+5*MAX(2;MIN(5;INT(Income/B59/D59/0,2)*0,2)))/100)
 */
function calc_pd_norm(kz, income, min, family)
{
	if (kz <= 0.5) return 0.5;
	return (25 + 5 * Math.max(2, Math.min(5, Math.round(5 * income/ (min * family))*0.2)))/100;
}

/* 
 * Вычисление соотношения Платеж/Доход
 * Поля E58, расходы по другим кредитам, нет
 * XLS: =ROUND((Payment+E58)/Income;4)
 */
function calc_pd(payment, income) { return round4(payment/income); }


/* 
 * Вычисление норматива Кредит/Залог
 * Стоимость квартиры (Cost_1) считается ненулевой, поле C63 -- отсутсвующим
 * XLS: =IF(OR(Cost_1=0;Cost_1=C63);0;MIN(IF(KZ<=0,5;0,5;0,9);D63))
 */
function calc_kz_norm(kz)
{
	if (kz <= 0.5) return 0.5;
	return 0.9;
}

/*
 * Вычисление отношения Кредит/Залог
 * Считается что задана только одна стоимость
 * XLS: =ROUND(Amount/IF(Cost_2<>0;MIN(Cost_1;Cost_2);Cost_1);4)
 */
function calc_kz(amount, cost)
{
	//XXX: Здесь определена только одна стоимость
	return round4(amount/cost);
}

/*
 * Вычисления в режиме "От суммы кредита"
 * Входные поля - term, amount
 * Выходные - annual, payment
 */
function calc_mode_amount()
{
	// Form already validated
	var term = 12 * (document.getElementById('term').value);
	var amount = document.getElementById('amount').value;
	var annual = calc_annual(amount, term, c_rate);
	var insurance = calc_ins(amount, c_ins_rate);
	//document.getElementById('annual').value = annual;
	document.getElementById('insurance').value = insurance;
	document.getElementById('payment').value = round2(annual + insurance);
	return '';
}

/*
 * Вычисления в режиме "От суммы платежа"
 * Входные поля - term, payment 
 * Выходные - annual, amount 
 */
function calc_mode_payment()
{
	// Form already validated
	var term = 12 * document.getElementById('term').value;
	var payment = document.getElementById('payment').value;
	var amount = calc_rev_amount(payment, term, c_rate, c_ins_rate);
	var annual = calc_annual(amount, term, c_rate);
	document.getElementById('amount').value = amount;
	//document.getElementById('annual').value = annual;
	document.getElementById('insurance').value = calc_ins(amount, c_ins_rate);
	return '';
}

/*
 * Вычисления в режиме "Срок кредита"
 * Входные поля - amount, payment
 * Выходные - annual, term
 */
function calc_mode_term()
{
	// Form already validated
	var amount = document.getElementById('amount').value;
	var payment = document.getElementById('payment').value;
	var term = calc_rev_term(amount, payment, c_rate, c_ins_rate);
	var annual = calc_annual(amount, term, c_rate);
	document.getElementById('term').value = round2(term / 12);
	//document.getElementById('annual').value = annual;
	document.getElementById('insurance').value = calc_ins(amount, c_ins_rate);
	return '';
}

/*
 * Вычисления в режиме "От дохода"
 * Входные поля - term, income, family, cash 
 * Выходные - annual, amount, flat
 * Сначала производится расчет по двум крайним случаям
 * 1 - соотношение К/З максимально возможное (0.9)
 * 2 - соотношение К/З у границы 0.5 (в этом месте изменяется вычисление норматива П/Д)
 * Дальше сравнивается, верно ли соотношение кредита/собственных средств.
 * Если оно верно, то выбирается максимальный из возможных кредитов
 * Если нет, то расчет ведется от собственных средств
 */
function calc_mode_income()
{
	// Form already validated
	var error = "";
	var income = document.getElementById('income').value;
	var family = document.getElementById('family').value;
	var term = 12 * (document.getElementById('term').value);
	var cash = document.getElementById('cash').value;

	var pd_norm0 = calc_pd_norm(0.5, income, c_minimum, family);
	var payment0 = income * pd_norm0;
	var amount0 = calc_rev_amount(payment0, term, c_rate, c_ins_rate);

	var payment = payment0;
	var amount = amount0;

	var pd_norm1 = calc_pd_norm(0.9, income, c_minimum, family);
	var payment1 = income * pd_norm1;
	var amount1 = calc_rev_amount(payment1, term, c_rate, c_ins_rate);

	if (amount1 / 9 > 1. * cash && amount0 * 0.5 > cash) {
		error = "Расчет производится из размера собственных средств";
		amount = 9 * cash;
		var annual = calc_annual(amount, term, c_rate);
		payment = round2(annual + calc_ins(amount, 0.015));
	} else if (amount1 > amount0 || amount0 * 0.5 > cash) {
		payment = payment1;
		amount = amount1;
	}

	var kz = round4(amount / ((1. * cash) + amount));

	var annual = calc_annual(amount, term, c_rate);
	document.getElementById('payment').value = round2(payment);
	document.getElementById('amount').value = amount;
	//document.getElementById('annual').value = annual;
	document.getElementById('insurance').value = calc_ins(amount, c_ins_rate);
	document.getElementById('cost').value = round2(amount / kz);
	return error;
}
